#include <vector>
#include <time.h>
#include <Windows.h>
#include <iostream>

#include <fstream>

#include "interface.h"
#include "astrostuff.h"
#include "astro2.h"	

/**
* Constructor that initializes two names of planets arrays in two formats,
* the first with Capital first letter and the second with all caps,
* also i_maxplanets is set to how many items are in the planets arrays,
* and wait is set to 1200 ms for delay when sending information to the irc server
*/

Astro::Astro()
{


		pch_planets[0]="Sun";
		pch_planets[1]="Moon";
		pch_planets[2]="Mercury";
		pch_planets[3]="Venus";
		pch_planets[4]="Mars";
		pch_planets[5]="Jupiter";
		pch_planets[6]="Saturn";
		pch_planets[7]="Uranus";
		pch_planets[8]="Neptune";
		pch_planets[9]="Pluto";
		pch_planets[10]="Chiron";
		pch_planets[11]="Eris";
		pch_planets[12]="Pholus";
		pch_planets[13]="Ceres";
		pch_planets[14]="Pallas";
		pch_planets[15]="Juno";
		pch_planets[16]="Vesta";
		pch_planets[17]="N_Node";
		
		pch_planets2[0]="SUN";
		pch_planets2[1]="MOON";
		pch_planets2[2]="MERCURY";
		pch_planets2[3]="VENUS";
		pch_planets2[4]="MARS";
		pch_planets2[5]="JUPITER";
		pch_planets2[6]="SATURN";
		pch_planets2[7]="URANUS";
		pch_planets2[8]="NEPTUNE";
		pch_planets2[9]="PLUTO";
		pch_planets2[10]="CHIRON";
		pch_planets2[11]="ERIS";
		pch_planets2[12]="PHOLUS";
		pch_planets2[13]="CERES";
		pch_planets2[14]="PALLAS";
		pch_planets2[15]="JUNO";
		pch_planets2[16]="VESTA";
		pch_planets2[17]="N_NODE";
	
		i_maxplanets=18;
		wait=1200;


}


/**
* Calculates the positions of the planets for the show planets command, 
* This function is also called for calculateAspects and calculateTransits,
* and the double array of planet degrees is sent to these functions.
* if latitude and longitude are known the function will calculate ASC,
* a queryTime object is used to parse the tell sent to see what information is,
* being asked for.
*
* @param n  This would contain a name of the person making the query,
* and is not used in the irc version, mychannel is instead, but is used 
* on the chessclub version.
* @param p the content of the tell or message sent.
* @param direction tells what type of query is being done.
* !planets is 1, !chart is 2 !aspects is 5 and !transits is 6
* @param specific  tells if the query is specific or general. if false then 
* its just something like !planets or !aspects with no other arguments.
* @param mychannel a string that says who to send a response to,
* could be a nick or a room
* @param type the arguments inner, outer, asteroid and all can be used to 
* reprensent the set of planets that are wanted. type is 1-4 reprensenting 
* those 4 types, inner=1 and all=4 in that order. If no planet
* set arguments are used then type is 0 and the default minplanets=0 and
* maxplanets=11 is used.
* @return none
*/


void Astro::ircplanets( char *n, char *p, int direction, bool specific, char *mychannel, int type)
{
	queryTime qobject;
	char timebuf[256];
	char datebuf[256];
	_strtime( timebuf );
	_strdate(datebuf);
	time_t rawtime;
	tm * ptm;
	time ( &rawtime );
	ptm = gmtime ( &rawtime );
	double thetime= (double) (ptm->tm_hour+(double)ptm->tm_min/60);
	
	double planet_signs[25];
	int retrograde[25];
	int day=ptm->tm_mday;
	int month=ptm->tm_mon+1;
	int year=ptm->tm_year+1900;
	int major=1;

	char *temp=  new char[700]; temp[0]='\0';
	int weight=0;  // used to tell if asc info is available
	double cusp[16];  /* empty + 12 houses */

	if(specific==true)
	{
		type=qobject.readString(p, 0);

		if(type>=10 && type<15)
		{	
			major=0;
			type=type-10;
		}
	
		if(type==-1 && direction==6)
			return;

		if(type!=-1)
		{
			day=qobject.day;
			month=qobject.month;
			year=qobject.year;
			thetime=qobject.thetime;
		}
		else
			return;	
	} // end if specific

	qobject.setMinMaxPlanets(type);
	double astro_time=swe_julday(year, month, day, thetime, 1);
	
	if(qobject.maxplanets==11 && qobject.minplanets==0)
		if(qobject.lon!=-1000)
		{
			weight=1;
			pch_planets[11]="ASC";
			calculateASC(astro_time, temp, cusp, qobject);	
		}
	
	if(direction==2 && weight==0)
		return;

	// calculate planet degrees here
	planetDegrees(astro_time, planet_signs, retrograde);

	for(int a=qobject.minplanets; a<qobject.maxplanets; a++)
		writeplanetstring(temp, planet_signs[a], a, weight, direction, cusp, qobject.maxplanets, retrograde[a]);
	

	if(weight==1)
		planet_signs[11]=cusp[1];

if(direction==1 || direction==2)
{
	Sendf("PRIVMSG %s :%s\r\n", mychannel, temp);
	printf("\n %s !planets %s %s specific=%d", n, timebuf, datebuf, specific);
	printf("\n %s", p);
	Sleep(wait);
}

if(direction==5)
	calculateAspects(planet_signs, qobject.minplanets, qobject.maxplanets, mychannel, major, weight);

if(direction==6)
	calculateTransits(planet_signs, qobject.minplanets, qobject.maxplanets, mychannel, weight, qobject);

if(weight==1)
	pch_planets[11]="Eris";

delete temp;			
}// end function

/**
* Function is passed a pointer to a string and the degree of the planet
* and writes a formated string in two formats depending on if 
* direction is 1 (!planets) or 2 ( !chart)
*
* @param temp  This is the string, which is passed by reference and shared by the function that 
* called, that the formated string is written to.
* @param xxx a double that represents the degree of the planet. Could also be 
* the degree of the ASC
* @param a  an int that represents the value of the planet in any arrays
* @param weight an int =1 if house info is available otherwise 0
* @param direction an int is 1 or 2 depending on if its !planets or !chart
* @param cusp an array of doubles that has house info for if direction is 2
* @param maxplanets an int that lets the function know if the planet is 
* the last planet in the overall string for differnet formating of the end 
* of the string
* @return none
*/

void Astro::writeplanetstring(char *temp, double xxx, int a, int weight, int direction, double cusp[], int maxplanets, int retro)
{
		char tempcharnum[33];
		char tempminute[33];
        convert(xxx, tempcharnum, tempminute);
		
		if(direction==1){
		
		temp=strcat(temp, pch_planets[a]);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempcharnum);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempminute);
		temp=strcat(temp, "' ");
		if(retro==1)
			temp=strcat(temp, "(R) ");

		temp=strcat(temp, getsign(xxx));
		if(a<maxplanets-1)
			temp=strcat(temp, "; ");
		}
		
		if(direction==2){

		temp=strcat(temp, pch_planets2[a]);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempcharnum);
		temp=strcat(temp, " ");
		temp=strcat(temp, getsign(xxx));
		temp=strcat(temp, " ");
		temp=strcat(temp, tempminute);
		temp=strcat(temp, "'");
		if(retro==1)
			temp=strcat(temp, " (R)");


		if(weight==1)
		{	char temp11[10];
			temp11[0]='\0';

			temp=strcat(temp, " H");
			temp=strcat(temp, gethouse(xxx, cusp, temp11));
		}
		if(a<maxplanets-1)
			temp=strcat(temp, "; ");

		
		}



}

/**
/* Function is used by ircplanets to do the calculating of the degrees of 
/* the planets.  Calculates twice and compares the two degrees to see
/* if the planet is moving forward or backward relative to the earth.  The leap 
/* array is the time differential for reach planet in doing the second calculation.
/* @param astro_time a double that represents the time for the calculation.
/* @param planet_signs and array of doubles that records the degree of 
/* the planets and is passed by reference so the calling function has access
/* to the array after its populated by calculation.
/* @param retrograde an array of ints that say if the planet is moving foward ( direct)
/* or backward , retrograde.
/* @return no return type
*/

void Astro::planetDegrees(double astro_time, double planet_signs[], int retrograde[])
{
	double xxx[6], *xx=xxx;
	char serr[999], *serr2=serr;
	int  val=0;
	double leap[25];
	for(int l=0; l<25; l++)
	{
		if(l>7)
			leap[l]=.12;
		if(l>3 && l<=7)
			leap[l]=.06;
		if(l<=3)
			leap[l]=.03;
	}
	for(int a=0; a<i_maxplanets; a++){	
		
			val=swe_calc_ut(astro_time,  a, 0, xx, serr2);
		if(a==10) 
			val=swe_calc_ut(astro_time,  SE_CHIRON, 0, xx, serr2);
		if(a==11)
			val=swe_calc_ut(astro_time,  SE_AST_OFFSET +  136199, 0, xx, serr2);
		if(a==12) 
			val=swe_calc_ut(astro_time,  SE_PHOLUS, 0, xx, serr2);
		if(a==13)
			val=swe_calc_ut(astro_time,  SE_CERES, 0, xx, serr2);
		if(a==14) 
			val=swe_calc_ut(astro_time,  SE_PALLAS, 0, xx, serr2);
		if(a==15)
			val=swe_calc_ut(astro_time,  SE_JUNO, 0, xx, serr2);
		if(a==16) 
			val=swe_calc_ut(astro_time,  SE_VESTA, 0, xx, serr2);
		if(a==17)
			val=swe_calc_ut(astro_time,  SE_TRUE_NODE, 0, xx, serr2);

		

		planet_signs[a]=xxx[0];
		
		// now see if its retrograde
		val=swe_calc_ut(astro_time+leap[a],  a, 0, xx, serr2);
		if(a==10) 
			val=swe_calc_ut(astro_time+leap[a],  SE_CHIRON, 0, xx, serr2);
		if(a==11)
			val=swe_calc_ut(astro_time+leap[a],  SE_AST_OFFSET +  136199, 0, xx, serr2);
		if(a==12) 
			val=swe_calc_ut(astro_time+leap[a],  SE_PHOLUS, 0, xx, serr2);
		if(a==13)
			val=swe_calc_ut(astro_time+leap[a],  SE_CERES, 0, xx, serr2);
		if(a==14) 
			val=swe_calc_ut(astro_time+leap[a],  SE_PALLAS, 0, xx, serr2);
		if(a==15)
			val=swe_calc_ut(astro_time+leap[a],  SE_JUNO, 0, xx, serr2);
		if(a==16) 
			val=swe_calc_ut(astro_time+leap[a],  SE_VESTA, 0, xx, serr2);
		if(a==17)
			val=swe_calc_ut(astro_time+leap[a],  SE_TRUE_NODE, 0, xx, serr2);

		if(xxx[0]<planet_signs[a])
			retrograde[a]=1;
		else
			retrograde[a]=0;

	} // end for
} // end function

/**
/* Function is also called by ircplanets to find the rising sign at the time 
/* in question.
/* @param astro_time the time to calculate for. A double value.
/* @param temp a char * pointer that holds the textual representation of the rising.
/* its passed by reference so the calling function has access to this string.
/* @param cusp an array of doubles that in the cusp[1] spot holds the rising.
/* @param qobject the querytime object that holds the latitude and longitude for 
/* the calculation.
/* @return none
*/
 
void Astro::calculateASC(double astro_time, char *temp, double cusp[], queryTime qobject)
{


	double ascmc[10];    /* asc, mc, armc, vertex .. */
	int aaa=swe_houses(astro_time,  (double ) qobject.lat, (double ) qobject.lon, qobject.system, cusp,  ascmc);
	char tempcharnum2[33];
	char tempminute2[33];
	convert(cusp[1], tempcharnum2, tempminute2);
	temp=strcat(temp, "Asc");
	temp=strcat(temp, " ");
	temp=strcat(temp, tempcharnum2);
	temp=strcat(temp, " ");
	temp=strcat(temp, tempminute2);
	temp=strcat(temp, "' ");
	temp=strcat(temp, getsign(cusp[1]));
	temp=strcat(temp, "; ");

}

/**
* This function is used by writeplanetstring to find the house that  planet is 
* in for the !chart command.  It converts the house from a number to a string
* and returns the string.
*
* @param sign a double that is the degree of the planet
* @param cusp an array of doubles that reprenent the house cusps or border points and 
* the house is the one in which sign lies between the beginning of a house and end.
* @param temp1 passed by reference and is returned to be the formated string this 
* function produces.
* @return returns a formated string with the house number such as "1" or "1/2" if 
* the planet is at the end of a house.
*/

char *Astro::gethouse(double sign, double cusp[], char temp1[])
{
		int a, b, c;
		char temp22[10], *temp2=temp22;
		temp22[0]='\0';

		
		for(a=0; a<11; a++)
		{
			if(sign>=cusp[a+1] && sign <cusp[a+2])
			{	temp1=itoa(a+1, temp1, 10);
				if(sign>cusp[a+2]-3.5)
					// its on the cusp
				{
					temp2=itoa(a+2, temp2, 10);
					for(b=0; temp1[b]!='\0'; b++)
						; // just move through
					temp1[b++]='/';
					for(c=b; temp2[c-b]!='\0'; c++)
						temp1[c]=temp2[c-b];
					temp1[c]='\0';
					
				}
				return temp1;
			}
		}


			if(sign>=cusp[12] && sign <cusp[1])
			{	
				if(sign>cusp[1]-3.5 && sign<cusp[1])
					// its on the cusp
					return "12/1";
				
				return "12";
			}


			// its an aries sign
			for(a=2; a<13; a++)
			{
			if(cusp[a]<cusp[a-1])
			{	temp1=itoa(a-1, temp1, 10);
				if(sign>cusp[a]-3.5)
					// its on the cusp
				{
					temp2=itoa(a, temp2, 10);
					for(b=0; temp1[b]!='\0'; b++)
						; // just move through
					temp1[b++]='/';
					for(c=b; temp2[c-b]!='\0'; c++)
						temp1[c]=temp2[c-b];
					temp1[c]='\0';
					
				}
				return temp1;
			}
				}

		if(sign>cusp[1]-3.5)
			return "12/1";
		else
			return "12";

	return " ";

}

/**
* This function converts a double that represents the degree into a string of hour and 
* minute format such as  6 and 29'
*
* @param myDegree a double that is the degree to be converted
* @param tempcharnum a char array passed by reference that holds the degree
* @param tempminute a char aray passed by refence that holds the minute
* @return none
*/
	
void Astro::convert(double myDegree, char tempcharnum[], char tempminute[])
{

		// degreetypes[a]=xxx[0];
		
		int s= (int)((myDegree/30))%12 + 1;
		// ss[a]=s;
		int tempnum=(int) (myDegree-((s-1)*30));
		itoa(tempnum, tempcharnum, 10);

		int degree= (int) myDegree;
		double fraction= ( double) myDegree-degree;
		double fraction2= (double) fraction*3600;

		double minute1=fraction2/60;
		int minute= (int ) (fraction2/60);
		int rounding=10 * (double) ( minute1- (double) minute);
		if(rounding>=5)
			minute++;
		// minutes[a]=minute;
		itoa(minute, tempminute, 10);
}

/**
* this function writes a formated aspects string such as  Sun Sext Pluto 1 24' for
* the function calculateAspects
*
* @param temp a pointer to a char array that holds the string and is passed by reference hence 
* no return type is needed
* @param tempcharnum[] holds the degree that is appended to temp
* @param tempminute[] holds the minute that is appended to temp
* @param type and int with type of aspect such as 0 = "Conj"
* @param i  number of the first planet aspecting used to index an array to find the 
* string with the planets name
* @param j same as i except its the second planet
* @return none
*/

void Astro::makeAspectString(char *temp, char tempcharnum[], char tempminute[], int type, int i, int j)
{
		char *types[10];
		types[0]="Conj";
		types[1]="Trine";
		types[2]="Square";
		types[3]="Opp";
		types[4]="Sext";
		types[5]="SemiSextile";
		types[6]="Decile";
		types[7]="SemiSquare";


		temp=strcat(temp, pch_planets[i]);
		temp=strcat(temp, " ");
		temp=strcat(temp, types[type]);
		temp=strcat(temp, " ");
		temp=strcat(temp, pch_planets[j]);
		temp=strcat(temp, " ");

		temp=strcat(temp, tempcharnum);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempminute);
		temp=strcat(temp, "' ");
		
}
/**
* this function writes a formated transits string such as  Mercury Sext nMoon 0 50' for
* the function calculateTransits. Similar to makeAspectString exept the format is slightly 
* different and gives freedom to use a different format.  One example of a difference is 
* the order of planet i and planet j are inverted in the string from make aspect string.
*
* @param temp a pointer to a char array that holds the string and is passed by reference hence 
* no return type is needed
* @param tempcharnum[] holds the degree that is appended to temp
* @param tempminute[] holds the minute that is appended to temp
* @param type and int with type of aspect such as 0 = "Conj"
* @param i  number of the first planet aspecting used to index an array to find the 
* string with the planets name
* @param j same as i except its the second planet
* @return none
*/

void Astro::makeTransitString(char *temp, char tempcharnum[], char tempminute[], int type, int i, int j)
{
		char *types[10];
		types[0]="Conj";
		types[1]="Trine";
		types[2]="Square";
		types[3]="Opp";
		types[4]="Sext";
		
		temp=strcat(temp, pch_planets[j]);
		temp=strcat(temp, " ");
		temp=strcat(temp, types[type]);
		temp=strcat(temp, " n");
		temp=strcat(temp, pch_planets[i]);
		temp=strcat(temp, " ");

		temp=strcat(temp, tempcharnum);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempminute);
		temp=strcat(temp, "' ");
		
}

/**
* This function finds out what the sign is for a given degree and returns it.
*
* @param truedegree the degree to find the sign for
* @return a string that represents the sign such as "Ari"
*/

char *Astro::getsign(double truedegree)
{
		
		int  s= (int)((truedegree/30))%12 + 1;
		
		

		if(s==1)
			return "Ari";
		if(s==2)
			return "Tau";
		if(s==3)
			return "Gem";
		if(s==4)
			return "Can";
		if(s==5)
			return "Leo";
		if(s==6)
			return "Vir";


		if(s==7)
			return "Lib";
		if(s==8)
			return "Sco";
		if(s==9)
			return "Sag";
		if(s==10)
			return "Cap";
		if(s==11)
			return "Aqu";
		if(s==12)
			return "Pis";

		return "";
}


/**
* Calculates the aspects between the planets in a set of planets calculated for a given date 
* then sends the output to whoever asked. Aspects are a fixed number of degrees between two 
* planets such as a square is 90 degrees between two planets. An orb is allowed so a 6 degree 
* orb would find a square if it finds two planets are 84-96 degrees apart.
* 
* @param planet_signs a double with the degrees of all the planets in the set
* @param pbottom an int that indicates the minimum planet in the range of the array
* of planets calculated. 
* @param ptop an int that indicates the maximum planet number in the array of 
* planets to be calculated.  So pbottom 0 and ptop = 5 would calculate 6 planets 
* numbers 0-5 in the array of planets.
* @param mychannel a string that says who to send a response to,
* could be a nick or a room
* @param major an int that if 0 tells it to calculate the minor aspects otherwise
* the function will calculate major aspects.
* @param weight an int that if 1 tells it to calculate the rising as an aspect
* because house info is available.  Only kicks in if the default pbottom and ptop is used 
* 0 and 11
* @return none
*/

void Astro::calculateAspects(double planet_signs[], int pbottom, int ptop, char *mychannel, int major, int weight)
{

double conjunction=8;
double square=8;
double trine=8;
double opposition=8;
double sextile=5;
double orb=0;
double semisquare=1.5;
double semisextile=1.5;
double decile=1.5;
int maxstrlen=300;
char *temp=  new char[500]; temp[0]='\0';
char *temp2=  new char[500]; temp2[0]='\0';
char *temp3=  new char[500]; temp3[0]='\0';
char *temp4=  new char[500]; temp4[0]='\0';
char *temp5=  new char[500]; temp5[0]='\0';
int mymax=11;

if(pbottom==0 && ptop==11)
	mymax=11+weight;
else
	mymax=i_maxplanets;

planetcheck obj;
 

for(int ii=pbottom; ii<ptop; ii++){

	for(int jj=0; jj<mymax; jj++)
	{
	char tempcharnum[33];
	char tempminute[33];
	if(obj.check(ii, jj, 0)==0)
		continue;

	if(major==1){
	// conjunctions
	if(planet_signs[ii]>planet_signs[jj])
		orb=planet_signs[ii]-planet_signs[jj];
	else
		orb=planet_signs[jj]-planet_signs[ii];
	if(360-orb<orb)
		orb=360-orb;

	if(orb<=conjunction)
	{
	
		convert(orb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 0, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 0, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 0, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 0, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 0, ii, jj);
		
			
	}
		double temporb=orb-120;
		if(temporb<0)
			temporb=temporb*-1;

		if(temporb<=trine)
		{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 1, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 1, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 1, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 1, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 1, ii, jj);
	
		}

		temporb=orb-90;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=square)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 2, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 2, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 2, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 2, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 2, ii, jj);
		
	}

		temporb=orb-180;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=opposition)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 3, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 3, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 3, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 3, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 3, ii, jj);
	
	}
		temporb=orb-60;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=sextile)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 4, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 4, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 4, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 4, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 4, ii, jj);
	
	}

	}// end if major

	else  // minor aspects
	{
	
	if(planet_signs[ii]>planet_signs[jj])
		orb=planet_signs[ii]-planet_signs[jj];
	else
		orb=planet_signs[jj]-planet_signs[ii];
	if(360-orb<orb)
		orb=360-orb;

		double temporb=orb-30;
		if(temporb<0)
			temporb=temporb*-1;

		if(temporb<=semisextile)
		{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 5, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 5, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 5, ii, jj);
			else
				if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 5, ii, jj);
				else
					if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 5, ii, jj);

		}

		temporb=orb-36;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=decile)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 6, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 6, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 6, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 6, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 6, ii, jj);

	}

		temporb=orb-45;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=semisquare)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeAspectString(temp, tempcharnum, tempminute, 7, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeAspectString(temp2, tempcharnum, tempminute, 7, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeAspectString(temp3, tempcharnum, tempminute, 7, ii, jj);
				else
					if(strlen(temp4)<maxstrlen) makeAspectString(temp4, tempcharnum, tempminute, 7, ii, jj);
					else
						if(strlen(temp5)<maxstrlen) makeAspectString(temp5, tempcharnum, tempminute, 7, ii, jj);

	}


	}

}// end inner for


}// end outer for
	printf("\ncalculate aspects");
	Sendf("PRIVMSG %s :%s\r\n", mychannel, temp);
	Sleep(wait);
	if(strlen(temp2)>1)
	{		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp2);
		Sleep(wait);
	}
	if(strlen(temp3)>1)
	{		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp3);
		Sleep(wait);
	}
	if(strlen(temp4)>1)
	{		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp4);
		Sleep(wait);
	}
	if(strlen(temp5)>1)
	{		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp5);
		Sleep(wait);
	}

	delete temp;
	delete temp2;
	delete temp3;
	delete temp4;
	delete temp5;

} // end function

/**
* A function simmilar to calculateAspects but calculates aspects between two different 
* sets of planets.  I.E. planets from two different dates.  The second set of planets is 
* by default the present position of the planets unless a second date was given with
* the !transits command.  Then it will find the transits between the first date and
* the second date.
*
* @param planet_signs an array of doubles that represents the planet degrees of the 
* first set of planets.
* @param pbottom an int that indicates the minimum planet in the range of the array
* of planets calculated. 
* @param ptop an int that indicates the maximum planet number in the array of 
* planets to be calculated.  So pbottom 0 and ptop = 5 would calculate 6 planets 
* numbers 0-5 in the array of planets.
* @param mychannel a string that says who to send a response to,
* could be a nick or a room
* @param weight an int that if 1 tells it to calculate transits to the rising
* because house info is available.  
* @param qobject a queryTime object that Caclulateplanets used to find the first set of planets 
* and is checked here to see if a second set of planets was indicated.
* @return none
*/

void Astro::calculateTransits(double planet_signs[], int pbottom, int ptop, char *mychannel, int weight, queryTime qobject)
{

double conjunction=2;
double square=2;
double trine=2;
double opposition=2;
double sextile=2;
double orb=2;
char *temp=  new char[500]; temp[0]='\0';
char *temp2=  new char[500]; temp2[0]='\0';
char *temp3=  new char[500]; temp3[0]='\0';
int mymax=11;
int maxstrlen=300;


	time_t rawtime;
	tm * ptm;
	time ( &rawtime );
	ptm = gmtime ( &rawtime );
	double thetime= (double) (ptm->tm_hour+(double)ptm->tm_min/60);
	int day=ptm->tm_mday;
	int month=ptm->tm_mon+1;
	int year=ptm->tm_year+1900;

	if(qobject.year2!=-1000)
		if(qobject.month2!=0)
			if(qobject.day2!=0)
	{
		day=qobject.day2;
		month=qobject.month2;
		year=qobject.year2;
		thetime=qobject.thetime2;
	}

if(pbottom==0 && ptop==11)
	mymax=11;
else
	mymax=i_maxplanets;


	double astro_time=swe_julday(year, month, day, thetime, 1);
	double planet_signs2[25];
	int retrograde[25];
	planetDegrees(astro_time, planet_signs2, retrograde);

int jtop=11;
if(ptop==i_maxplanets)
	jtop=i_maxplanets;

int mynatalmax=mymax+weight;
for(int ii=0; ii<mynatalmax; ii++){

	for(int jj=0; jj<mymax; jj++)
	{
	char tempcharnum[33];
	char tempminute[33];
	bool doit=false;
	if(jj>=pbottom && jj<ptop)
		doit=true;
	if(ii>=pbottom && jj<ptop)
		doit=true;
	if(!doit)
		continue;

	// conjunctions
	if(planet_signs[ii]>planet_signs2[jj])
		orb=planet_signs[ii]-planet_signs2[jj];
	else
		orb=planet_signs2[jj]-planet_signs[ii];
	if(360-orb<orb)
		orb=360-orb;

	if(orb<=conjunction)
	{
	
		convert(orb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeTransitString(temp, tempcharnum, tempminute, 0, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeTransitString(temp2, tempcharnum, tempminute, 0, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeTransitString(temp3, tempcharnum, tempminute, 0, ii, jj);

			
	}
		double temporb=orb-120;
		if(temporb<0)
			temporb=temporb*-1;

		if(temporb<=trine)
		{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeTransitString(temp, tempcharnum, tempminute, 1, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeTransitString(temp2, tempcharnum, tempminute, 1, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeTransitString(temp3, tempcharnum, tempminute, 1, ii, jj);
	
		}

		temporb=orb-90;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=square)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeTransitString(temp, tempcharnum, tempminute, 2, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeTransitString(temp2, tempcharnum, tempminute, 2, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeTransitString(temp3, tempcharnum, tempminute, 2, ii, jj);

	}

		temporb=orb-180;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=opposition)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeTransitString(temp, tempcharnum, tempminute, 3, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeTransitString(temp2, tempcharnum, tempminute, 3, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeTransitString(temp3, tempcharnum, tempminute, 3, ii, jj);

	}
		temporb=orb-60;
		if(temporb<0)
			temporb=temporb*-1;

	if(temporb<=sextile)
	{
		convert(temporb, tempcharnum, tempminute);
		if(strlen(temp)<maxstrlen)
			makeTransitString(temp, tempcharnum, tempminute, 4, ii, jj);
		else
			if(strlen(temp2)<maxstrlen) makeTransitString(temp2, tempcharnum, tempminute, 4, ii, jj);
			else
				if(strlen(temp3)<maxstrlen) makeTransitString(temp3, tempcharnum, tempminute, 4, ii, jj);

	}

}// end inner for


}// end outer for

	printf("\ncalculate transits");
	Sendf("PRIVMSG %s :%s\r\n", mychannel, temp);
	Sleep(wait);
	if(strlen(temp2)>1)
	{		
		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp2);
		Sleep(wait);
	}
		if(strlen(temp3)>1)
	{		
		
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp3);
		Sleep(wait);
	}

delete temp;
delete temp2;
delete temp3;

} // end function

/** 
* A function that calculates the houses for a given date/time/lattiude and longitude.
* a queryTime object is created and a tell is sent to the queryTimes readstring 
* function to find out if there is enough info to calculate houses. If so 
* then this function makes the calculation and sends a response to whoever asked, 
* a room or person.
*
* @param n a char string, can be the name but is used in the icc version not the irc version currently
* @param p a char string that represents what was typed that invoked this function. Is
* the queryTime object converts the string to the values needed.
* @param mychannel a string that says who to send a response to,
* could be a nick or a room
* @return none
*/

void Astro::writeHouses( char *n, char *p, char *mychannel, char system)
{
	char hnum[50];
	char *temp=  new char[500]; temp[0]='\0';
	double cusp[16];  /* empty + 12 houses */
	double ascmc[10];    /* asc, mc, armc, vertex ... */
	queryTime qobject;

	int go=qobject.readString(p, 1);

	if(go==-1)
		return;
	double thetime= ( double ) (qobject.hour+ (double) qobject.minute/60);
	double astro_time=swe_julday(qobject.year, qobject.month, qobject.day, thetime, 1);

	char *hsy="PKORCAEVXHTBG";
	int aa=swe_houses(astro_time,  (double ) qobject.lat, (double ) qobject.lon, system, cusp,  ascmc);
	for (int ii = 1; ii <= 12; ii++)  {
		char tempcharnum[33];
		char tempminute[33];
        convert(cusp[ii], tempcharnum, tempminute);
		if(ii!=1 && ii!=10)
			temp=strcat(temp, "H");
		itoa(ii, hnum, 10);
		if(ii!=1 && ii!=10)
			temp=strcat(temp, hnum);
		if(ii==1)
			temp=strcat(temp, "Asc");
		if(ii==10)
			temp=strcat(temp, "MC");
		temp=strcat(temp, " ");
		temp=strcat(temp, tempcharnum);
		temp=strcat(temp, " ");
		temp=strcat(temp, tempminute);
		temp=strcat(temp, "' ");
		temp=strcat(temp, getsign(cusp[ii]));
		if(ii<12)
			temp=strcat(temp, "; ");

	
	} // end for
	
		Sendf("PRIVMSG %s :%s\r\n", mychannel, temp);
		Sleep(wait);
		printf("\nwrite houses");
	
	delete temp;

} // end function


